#include <stdio.h>
#include <stdlib.h>

#define MAX 19

long long Factorial(long long m)
{
	long long res=1;
	long long i;
	for(i=1; i<=m; i++)
		res = res * i;
	return res;
}

//Get stirling number with iteration method
//S2(n,m)=mS2(n-1,m)+S2(n-1,m-1)   (n>1,m>1)
//S2(n,0)=0S2(n,1)=1S2(n,n)=1S2(n,k)=0(k>n)
long long StirlingNumber_Iter(long long n, long long m)
{
	long long temp[MAX][MAX];
	long long i, j;

	for(i=0; i<=n; i++)
	{
		temp[i][0] = 0;
		temp[i][1] = 1;
		temp[i][i] = 1;
		for(j=i+1; j<=m; j++)
			temp[i][j] = 0;
	}
	for(i=0; i<=n; i++)
	{
		for(j=2; j<i; j++)
		{
			temp[i][j] = j * temp[i-1][j] + temp[i-1][j-1];
		}
	}
	return temp[n][m];
}

//Get stirling number with expression method
long long StirlingNumber_Expr(long long n, long long m)
{
	long long right = 0;
	long long left;
	long long temp;
	long long swap1, swap2, swap3;
	long long h;
	long long j;
	long long res;
	
	if(m > n)
		return 0;

	left = Factorial(m);
	for(h=0; h<=m; h++)
	{
		temp = m - h;
		swap1 = 1;
		for(j=1; j<=n; j++)
			swap1 = swap1 * temp;
		swap2 = 1;
		for(j=1; j<=h; j++)
			swap2 = swap2 * (-1);
		swap3 =  Factorial(m)/(Factorial(h)*Factorial(temp));
		right += swap1 * swap2 * swap3;
	}
	res = right/left;
	return res;
}

int main()
{
	long long n, m;
	int type;

	printf("---------- Second-Class Stirling Number ----------\n");
	printf("This program can calculate second-class stirling number less than 18\n");
	printf("1 -- Iterative method\n");
	printf("2 -- Expression method\n");
	printf("0 -- Exit\n\n");
	while(1)
	{
		printf("Input the option: ");
		scanf("%d", &type);
		if(type == 0)
		{
			printf("Thanks!\n");
			break;
		}
		else if(type == 1 || type == 2)
		{
			printf("Input n: ");
			scanf("%lld", &n);
			printf("Input m: ");
			scanf("%lld", &m);
			if(n >= 1 && n < MAX && m >= 1 && m < MAX)
			{
				if(type == 1)
				{
					printf("S(%lld,%lld) = %lld.\n\n", n, m, StirlingNumber_Iter(n, m));
				}
				else if(type == 2)
				{
					printf ("S(%lld,%lld) = %lld.\n\n", n, m, StirlingNumber_Expr(n, m));
				}
			}
			else
				printf("Illegal input!\n\n");
		}
		else
			printf("Illegal input!\n\n");
	}
	system("PAUSE");
	return 0;
}